Package: trajectories
Version: 0.2-10
Title: Classes and Methods for Trajectory Data
Authors@R: c(person("Edzer", "Pebesma", role = c("aut", "cre"), email = "edzer.pebesma@uni-muenster.de", comment = c(ORCID = "0000-0001-8049-7069")),
	person("Benedikt", "Klus", role = "aut"),
	person("Benedikt", "Graeler", role = "ctb"),
	person("Nikolai", "Gorte", role = "ctb"),
	person("Mehdi", "Moradi", role = "aut"))
Depends: R (>= 3.0.0)
Imports: stats, utils, graphics, methods, lattice, sp (>= 1.1-0),
        spacetime (>= 1.0-0), zoo
Suggests: OpenStreetMap, RCurl, rjson, adehabitatLT, xts, knitr, rgl,
        forecast, MASS, spatstat (>= 2.0-1), spatstat.explore,
        spatstat.geom, taxidata, sf, XML, devtools, jsonlite, ks, maps
Description: Classes and methods for trajectory data, with support for nesting individual Track objects in track sets (Tracks) and track sets for different entities in collections of Tracks. Methods include selection, generalization, aggregation, intersection, simulation, and plotting.
License: GPL (>= 2)
URL: https://github.com/edzer/trajectories
Additional_repositories: https://cran.uni-muenster.de/pebesma/
BugReports: https://github.com/edzer/trajectories/issues
VignetteBuilder: knitr
Collate: Class-Tracks.R Tracks-methods.R generalize.R stcube.R stplot.R
        difftrack.R compare-methods.R rtracks.R Trackstat.R
RoxygenNote: 6.0.1
NeedsCompilation: no
Author: Edzer Pebesma [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8049-7069>),
  Benedikt Klus [aut],
  Benedikt Graeler [ctb],
  Nikolai Gorte [ctb],
  Mehdi Moradi [aut]
Maintainer: Edzer Pebesma <edzer.pebesma@uni-muenster.de>
Packaged: 2026-02-13 20:12:35 UTC; edzer
Repository: CRAN
Date/Publication: 2026-02-14 16:50:02 UTC
Built: R 4.6.0; ; 2026-04-26 00:57:10 UTC; unix
