import(lattice)
import(methods)
import(sp)
import(spacetime)
importFrom(stats, approx, as.formula, na.omit, quantile, rnorm, arima.sim,
	rpois, runif, density)
importFrom(utils, head, tail, stack, unstack)
importFrom(graphics, arrows, box, lines, points, segments, legend, polygon, title)
importFrom(grDevices, rainbow, heat.colors)
importFrom(zoo, "index<-")
# importFrom(rgl, open3d, plot3d, lines3d)

exportClasses(
	Track,
	Tracks,
	TracksCollection,
	segments,
	difftrack
)

export(
	Track, Tracks, TracksCollection,
	TrackStats,
	TrackSummary,
	TracksSummary,
	segPanel,
	tracksPanel,
	frechetDist,
	approxTrack, approxTracks, approxTracksCollection,
	rTrack, rTracks, rTracksCollection, as.Track, as.Track.arrow,
	as.Track.ppp, auto.arima.Track, avedistTrack, avemove, chimaps,
	Kinhom.Track, pcfinhom.Track, reTrack, Track.idw, tsqTracks 
)

exportMethods(
	compare,
	dists,
	downsample,
	generalize,
	plot,
	stbox,
	stcube,
	spTransform,
	stplot
)

S3method(as.list, Tracks)
S3method(as.list, TracksCollection)
S3method(c, Track)
S3method(c, Tracks)
S3method(c, TracksCollection)
S3method(cut, Track)
S3method(cut, Tracks)
S3method(cut, TracksCollection)
S3method(density, list)
S3method(dim, Track)
S3method(dim, Tracks)
S3method(dim, TracksCollection)
S3method("index<-", Track)
S3method(plot, KTrack)
S3method(plot, distrack)
S3method(plot, gTrack)
S3method(plot, arwlen)
S3method(print, summary.Track)
S3method(print, summary.Tracks)
S3method(print, summary.TracksCollection)
S3method(print, arwlen)
S3method(print, distrack)
S3method(print, gTrack)
S3method(print, KTrack)
S3method(print, Trrow)
S3method(print, ArimaTrack)
S3method(range, Track)
S3method(unique, Track)
S3method(range, Track)
S3method(stack, TracksCollection)
S3method(unstack, TracksCollection)
